'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var evaluator = require('@astroapps/evaluator');
var language = require('@codemirror/language');
var highlight = require('@lezer/highlight');
var lint = require('@codemirror/lint');

const EvalLanguage = language.LRLanguage.define({
    parser: evaluator.parser.configure({
        props: [
            language.indentNodeProp.add({
                Application: language.delimitedIndent({ closing: ")", align: false }),
            }),
            language.foldNodeProp.add({
                Application: language.foldInside,
            }),
            highlight.styleTags({
                Reference: highlight.tags.definition(highlight.tags.variableName),
                ConstantLiteral: highlight.tags.bool,
                String: highlight.tags.string,
                Number: highlight.tags.number,
                "( )": highlight.tags.paren,
            }),
        ],
    }),
    languageData: {
        commentTokens: { line: ";" },
    },
});
const syntaxLinter = lint.linter((view) => {
    let diagnostics = [];
    language.syntaxTree(view.state)
        .cursor()
        .iterate((node) => {
        if (node.type.isError)
            diagnostics.push({
                from: node.from,
                to: node.to,
                severity: "error",
                message: "Syntax error",
            });
    });
    return diagnostics;
});
const isLetter = /^[a-zA-Z]/;
function evalCompletions(getCheckEnv) {
    return (context) => {
        let word = context.matchBefore(/\$?\w*/);
        if (!word || (word.from == word.to && !context.explicit))
            return null;
        const outAst = evaluator.parseEval(context.state.sliceDoc(0, word.from));
        const { env: { vars, dataType }, } = evaluator.typeCheck(getCheckEnv(), outAst);
        const dataCompletions = evaluator.isObjectType(dataType)
            ? Object.entries(dataType.fields).map(([label, t]) => ({
                label,
                type: "text",
                detail: "(" + t.type + ")",
                boost: 1,
            }))
            : [];
        const varCompletions = word.text.startsWith("$")
            ? Object.entries(vars)
                .filter((x) => x[0].match(isLetter))
                .map(([label, t]) => ({
                label: "$" + label,
                type: evaluator.isFunctionType(t) ? "function" : "variable",
                detail: "(" + t.type + ")",
            }))
            : [];
        return {
            from: word.from,
            options: [...dataCompletions, ...varCompletions],
        };
    };
}
function Evaluator() {
    return new language.LanguageSupport(EvalLanguage, syntaxLinter);
}

exports.EvalLanguage = EvalLanguage;
exports.Evaluator = Evaluator;
exports.evalCompletions = evalCompletions;
