import { parser, parseEval, typeCheck, isObjectType, isFunctionType } from '@astroapps/evaluator';
import { LRLanguage, indentNodeProp, delimitedIndent, foldNodeProp, foldInside, syntaxTree, LanguageSupport } from '@codemirror/language';
import { styleTags, tags } from '@lezer/highlight';
import { linter } from '@codemirror/lint';

const EvalLanguage = LRLanguage.define({
    parser: parser.configure({
        props: [
            indentNodeProp.add({
                Application: delimitedIndent({ closing: ")", align: false }),
            }),
            foldNodeProp.add({
                Application: foldInside,
            }),
            styleTags({
                Reference: tags.definition(tags.variableName),
                ConstantLiteral: tags.bool,
                String: tags.string,
                Number: tags.number,
                "( )": tags.paren,
            }),
        ],
    }),
    languageData: {
        commentTokens: { line: ";" },
    },
});
const syntaxLinter = linter((view) => {
    let diagnostics = [];
    syntaxTree(view.state)
        .cursor()
        .iterate((node) => {
        if (node.type.isError)
            diagnostics.push({
                from: node.from,
                to: node.to,
                severity: "error",
                message: "Syntax error",
            });
    });
    return diagnostics;
});
const isLetter = /^[a-zA-Z]/;
function evalCompletions(getCheckEnv) {
    return (context) => {
        let word = context.matchBefore(/\$?\w*/);
        if (!word || (word.from == word.to && !context.explicit))
            return null;
        const outAst = parseEval(context.state.sliceDoc(0, word.from));
        const { env: { vars, dataType }, } = typeCheck(getCheckEnv(), outAst);
        const dataCompletions = isObjectType(dataType)
            ? Object.entries(dataType.fields).map(([label, t]) => ({
                label,
                type: "text",
                detail: "(" + t.type + ")",
                boost: 1,
            }))
            : [];
        const varCompletions = word.text.startsWith("$")
            ? Object.entries(vars)
                .filter((x) => x[0].match(isLetter))
                .map(([label, t]) => ({
                label: "$" + label,
                type: isFunctionType(t) ? "function" : "variable",
                detail: "(" + t.type + ")",
            }))
            : [];
        return {
            from: word.from,
            options: [...dataCompletions, ...varCompletions],
        };
    };
}
function Evaluator() {
    return new LanguageSupport(EvalLanguage, syntaxLinter);
}

export { EvalLanguage, Evaluator, evalCompletions };
